<?php

// Sort types
/** @constant integer GOLLEM_SORT_TYPE  Sort by file type. */ define('GOLLEM_SORT_TYPE', 0);
/** @constant integer GOLLEM_SORT_NAME  Sort by file name. */ define('GOLLEM_SORT_NAME', 1);
/** @constant integer GOLLEM_SORT_DATE  Sort by file date. */ define('GOLLEM_SORT_DATE', 2);
/** @constant integer GOLLEM_SORT_SIZE  Sort by file size. */ define('GOLLEM_SORT_SIZE', 3);

// Sort direction
/** @constant integer GOLLEM_SORT_ASCEND  Sort ascending. */ define('GOLLEM_SORT_ASCEND', 0);
/** @constant integer GOLLEM_SORT_DESCEND  Sort descending. */ define('GOLLEM_SORT_DESCEND', 1);

/**
 * Gollem Base Class.
 *
 * $Horde: gollem/lib/Gollem.php,v 1.140 2004/12/20 03:35:06 slusarz Exp $
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Max Kalika <max@horde.org>
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @version $Revision: 1.140 $
 * @package Gollem
 */
class Gollem {

    /**
     * Check Gollem authentication and change to the currently active
     * directory. Redirects to login page on authentication/session failure.
     *
     * @access public
     *
     * @param optional boolean $redirect  Redirect to the logout page if
     *                                    authentication is unsuccessful?
     * @param optional string $mode       The authentication mode we are using.
     *
     * @return boolean  True on success, false on failure.
     */
    function checkAuthentication($redirect = true, $mode = null)
    {
        $auth_gollem = &Auth::singleton(array('gollem', 'gollem'));
        $reason = $auth_gollem->authenticate();

        if ($reason !== true) {
            if ($redirect) {
                if ($mode = 'selectlist') {
                    $url = Util::addParameter($GLOBALS['registry']->get('webroot', 'gollem') . '/login.php', 'selectlist_login', 1);
                } else {
                    $url = Auth::addLogoutParameters(Gollem::logoutUrl());
                }
                $url = Util::addParameter($url, 'url', Horde::selfUrl(true), false);
                header('Location: ' . $url);
                exit;
            } else {
                return false;
            }
        }

        return true;
    }

    /**
     * Can we log in without a login screen for the requested backend key?
     *
     * @param optional string $key     The backend key to check. Defaults to
     *                                 Gollem::getPreferredBackend().
     * @param optional boolean $force  If true, check the backend key even if
     *                                 there is more than one backend.
     *
     * @return boolean  True if autologin possible, false if not.
     */
    function canAutoLogin($key = null, $force = false)
    {
        require GOLLEM_BASE . '/config/backends.php';

        $auto_server = Gollem::getPreferredBackend();
        if (is_null($key)) {
            $key = $auto_server;
        }

        return (((count($auto_server) == 1) || $force) &&
                Auth::getAuth() &&
                !empty($backends[$key]['hordeauth']));
    }

    /**
     * Changes the current directory of the Gollem session to the supplied
     * value.
     *
     * @access public
     *
     * @param string $dir  Directory name.
     *
     * @return mixed  True on Success, PEAR_Error on failure.
     */
    function setDir($dir)
    {
        if (!Gollem::verifyDir($dir)) {
            return PEAR::raiseError(_("Access denied: ") . $dir);
        }
        $_SESSION['gollem']['dir'] = $dir;

        Gollem::_setLabel();

        return true;
    }

    /**
     * Changes the current directory of the Gollem session based on the
     * 'dir' form field.
     *
     * @access public
     *
     * @return mixed  True on Success, PEAR_Error on failure.
     */
    function changeDir()
    {
        $dir = Util::getFormData('dir');
        str_replace('..', '', $dir);

        if ($dir !== null) {
            if (!Gollem::verifyDir($dir)) {
                return PEAR::raiseError("Access denied: " . $dir);
            }
            $_SESSION['gollem']['dir'] = $dir;
        }

        Gollem::_setLabel();

        return true;
    }

    /**
     * Get the root directory of the Gollem session.
     *
     * @access public
     *
     * @return string  The root directory.
     */
    function getRoot()
    {
        return $_SESSION['gollem']['root'];
    }

    /**
     * Get the home directory of the Gollem session.
     *
     * @access public
     *
     * @return string  The home directory.
     */
    function getHome()
    {
        return $_SESSION['gollem']['home'];
    }

    /**
     * Get the current directory of the Gollem session.
     *
     * @access public
     *
     * @return mixed  Current dir on success or null on failure.
     */
    function getDir()
    {
        return ($_SESSION['gollem']['label'] !== _("[home]")) ?
            $_SESSION['gollem']['dir'] :
            $_SESSION['gollem']['home'];
    }

    /**
     * Set the lable to use for the current page.
     *
     * @access private
     */
    function _setLabel()
    {
        if ($_SESSION['gollem']['dir'] == $_SESSION['gollem']['home']) {
            $_SESSION['gollem']['label'] = _("[home]");
        } else {
            $_SESSION['gollem']['label'] = $_SESSION['gollem']['dir'];
        }

        if (empty($_SESSION['gollem']['label'])) {
            $_SESSION['gollem']['label'] = '/';
        }
    }

    /**
     * Internal helper to sort directories first if pref set.
     *
     * @access private
     */
    function _sortDirs($a, $b)
    {
        /* Sort symlinks to dirs as dirs */
        $dira = ($a['type'] === '**dir') ||
            (($a['type'] === '**sym') && ($a['linktype'] === '**dir'));
        $dirb = ($b['type'] === '**dir') ||
            (($b['type'] === '**sym') && ($b['linktype'] === '**dir'));

        if ($GLOBALS['prefs']->getValue('sortdirsfirst')) {
            if ($dira && !$dirb) {
                return -1;
            } elseif (!$dira && $dirb) {
                return 1;
            }
        }
        return 0;
    }

    /**
     * Internal sorting function for 'date'.
     *
     * @access private
     */
    function _sortDate($a, $b)
    {
        $dirs = Gollem::_sortDirs($a, $b);
        if ($dirs) {
            return $dirs;
        }

        if ($a['date'] > $b['date']) {
            return $GLOBALS['prefs']->getValue('sortdir') ? -1 : 1;
        } elseif ($a['date'] === $b['date']) {
            return Gollem::_sortName($a, $b);
        } else {
            return $GLOBALS['prefs']->getValue('sortdir') ? 1 : -1;
        }
    }

    /**
     * Internal sorting function for 'size'.
     *
     * @access private
     */
    function _sortSize($a, $b)
    {
        $dirs = Gollem::_sortDirs($a, $b);
        if ($dirs) {
            return $dirs;
        }

        if ($a['size'] > $b['size']) {
            return $GLOBALS['prefs']->getValue('sortdir') ? -1 : 1;
        } elseif ($a['size'] === $b['size']) {
            return 0;
        } else {
            return $GLOBALS['prefs']->getValue('sortdir') ? 1 : -1;
        }
    }

    /**
     * Internal sorting function for 'type'.
     *
     * @access private
     */
    function _sortType($a, $b)
    {
        $dirs = Gollem::_sortDirs($a, $b);
        if ($dirs) {
            return $dirs;
        }

        if ($a['type'] === $b['type']) {
            return Gollem::_sortName($a, $b);
        } elseif ($a['type'] === '**dir') {
            return $GLOBALS['prefs']->getValue('sortdir') ? 1 : -1;
        } elseif ($b['type'] === '**dir') {
            return $GLOBALS['prefs']->getValue('sortdir') ? -1 : 1;
        } else {
            $res = strcasecmp($a['type'], $b['type']);
            return $GLOBALS['prefs']->getValue('sortdir') ? ($res * -1) : $res;
        }
    }

    /**
     * Internal sorting function for 'name'.
     *
     * @access private
     */
    function _sortName($a, $b)
    {
        $dirs = Gollem::_sortDirs($a, $b);
        if ($dirs) {
            return $dirs;
        }

        $res = strcasecmp($a['name'], $b['name']);
        return $GLOBALS['prefs']->getValue('sortdir') ? ($res * -1) : $res;
    }

    /**
     * List the current folder.
     *
     * @access public
     *
     * @param TODO
     *
     * @return TODO
     */
    function listFolder($dir)
    {
        $files = $GLOBALS['vfs']->listFolder($dir, isset($_SESSION['gollem']['params']['filter']) ? $_SESSION['gollem']['params']['filter'] : null, $GLOBALS['prefs']->getValue('show_dotfiles'));
        if (!is_a($files, 'PEAR_Error')) {
            switch ($GLOBALS['prefs']->getValue('sortby')) {
            case GOLLEM_SORT_TYPE:
                usort($files, array('Gollem', '_sortType'));
                break;

            case GOLLEM_SORT_NAME:
                usort($files, array('Gollem', '_sortName'));
                break;

            case GOLLEM_SORT_DATE:
                usort($files, array('Gollem', '_sortDate'));
                break;

            case GOLLEM_SORT_SIZE:
                usort($files, array('Gollem', '_sortSize'));
                break;
            }
        }

        return $files;
    }

    /**
     * TODO
     */
    function subdirectory($base, $dir)
    {
        if (empty($base)) {
            return $dir;
        } else {
            if (substr($base, -1) == '/') {
                return $base . $dir;
            } else {
                return $base . '/' . $dir;
            }
        }
    }

    /**
     * Generate an URL to the logout screen that includes any known
     * information, such as username, server, etc., that can be filled
     * in on the login form.
     *
     * @access public
     *
     * @return string  The logout URL with parameters added.
     */
    function logoutUrl()
    {
        $params = array();
        $url = 'login.php';

        if (!empty($_SESSION['gollem']['params']['username'])) {
            $params['username'] = $_SESSION['gollem']['params']['username'];
        } elseif (Util::getFormData('username')) {
            $params['username'] = Util::getFormData('username');
        }

        if (!empty($_SESSION['gollem']['params']['port'])) {
            $params['port'] = $_SESSION['gollem']['params']['port'];
        }

        foreach ($params as $key => $val) {
            if (!empty($val)) {
                $url = Util::addParameter($url, $key, $val);
            }
        }

        return Horde::applicationUrl($url, true);
    }

    /**
     * Create a folder using the current Gollem session settings.
     *
     * @access public
     *
     * @param string $dir   TODO
     * @param string $name  TODO
     *
     * @return TODO
     */
    function createFolder($dir, $name)
    {
        if (!Gollem::verifyDir($dir)) {
            return PEAR::raiseError("Access denied: " . $dir);
        }
        return $GLOBALS['vfs']->createFolder($dir, $name);
    }

    /**
     * Rename files using the current Gollem session settings.
     *
     * @access public
     *
     * @param string $oldDir  TODO
     * @param string $old     TODO
     * @param string $newDir  TODO
     * @param string $old     TODO
     *
     * @return TODO
     */
    function renameItem($oldDir, $old, $newDir, $new)
    {
        return $GLOBALS['vfs']->rename($oldDir, $old, $newDir, $new);
    }

    /**
     * Delete a folder using the current Gollem session settings.
     *
     * @access public
     *
     * @param string $dir       TODO
     * @param string $name      TODO
     *
     * @return TODO
     */
    function deleteFolder($dir, $name)
    {
        if (!Gollem::verifyDir($dir)) {
            return PEAR::raiseError("Access denied: " . $dir);
        }
        if ($GLOBALS['prefs']->getValue('recursive_deletes') != 'disabled') {
            return $GLOBALS['vfs']->deleteFolder($dir, $name, true);
        }
        else {
            return $GLOBALS['vfs']->deleteFolder($dir, $name, false);
        }
    }

    /**
     * Delete a file using the current Gollem session settings.
     *
     * @access public
     *
     * @param string $dir   TODO
     * @param string $name  TODO
     *
     * @return TODO
     */
    function deleteFile($dir, $name)
    {
        if (!Gollem::verifyDir($dir)) {
            return PEAR::raiseError("Access denied: " . $dir);
        }
        return $GLOBALS['vfs']->deleteFile($dir, $name);
    }

    /**
     * Change permissions on files using the current Gollem session settings.
     *
     * @access public
     *
     * @param string $dir          TODO
     * @param string $name         TODO
     * @param integer $permission  TODO
     *
     * @return TODO
     */
    function changePermissions($dir, $name, $permission)
    {
        if (!Gollem::verifyDir($dir)) {
            return PEAR::raiseError("Access denied: " . $dir);
        }
        return $GLOBALS['vfs']->changePermissions($dir, $name, $permission);
    }

    /**
     * Write an uploaded file to the VFS backend.
     *
     * @access public
     *
     * @param string $dir       TODO
     * @param string $name      TODO
     * @param string $filename  TODO
     *
     * @return TODO
     */
    function writeFile($dir, $name, $filename)
    {
        return $GLOBALS['vfs']->write($dir, $name, $filename);

    }

    /**
     * Moves a file using the current Gollem session settings.
     *
     * @access public
     *
     * @param string $dir     TODO
     * @param string $name    TODO
     * @param string $newdir  TODO
     *
     * @return TODO
     */
    function moveFile($dir, $name, $newdir)
    {
        return $GLOBALS['vfs']->move($dir, $name, $newdir);
    }

    /**
     * Copies a file using the current Gollem session settings.
     *
     * @access public
     *
     * @param string $dir     TODO
     * @param string $name    TODO
     * @param string $newdir  TODO
     *
     * @return TODO
     */
    function copyFile($dir, $name, $newdir)
    {
        return $GLOBALS['vfs']->copy($dir, $name, $newdir);
    }

    /**
     * Get the current preferred backend key.
     *
     * @access public
     *
     * @return string  The preferred backend key.
     */
    function getPreferredBackend()
    {
        if (!empty($_SESSION['gollem']['backend_key'])) {
            $backend_key = $_SESSION['gollem']['backend_key'];
        } else {
            require GOLLEM_BASE . '/config/backends.php';
            /* Determine the preferred backend. */
            foreach ($backends as $key => $val) {
                if (empty($backend_key) && (substr($key, 0, 1) != '_')) {
                    $backend_key = $key;
                }
                if (!empty($val['preferred'])) {
                    $preferred = false;
                    if (!is_array($val['preferred'])) {
                        $val['preferred'] = array($val['preferred']);
                    }
                    foreach ($val['preferred'] as $backend) {
                        if (($backend == $_SERVER['SERVER_NAME']) ||
                            ($backend == $_SERVER['HTTP_HOST'])) {
                            $preferred = true;
                            break;
                        }
                    }
                    if ($preferred) {
                        $backend_key = $key;
                        break;
                    }
                }
            }
        }

        return $backend_key;
    }

    /**
     * This function verifies whether a given directory is below the root.
     *
     * @param string $dir  THe directory to check.
     *
     * @return boolean  True if the directory is below the root.
     */
    function verifyDir($dir)
    {
        $rootdir = Gollem::getRoot();
        return (String::substr($dir, 0, String::length($rootdir)) == $rootdir);
    }


    /**
     * TODO
     */
    function string2Columns($string)
    {
        $ret = array();
        $lines = explode("\n", $string);
        foreach ($lines as $line) {
            $line = trim($line);
            if (!empty($line)) {
                $columns = explode("\t", $line);
                if (count($columns) > 1) {
                    $source = array_splice($columns, 0, 1);
                    $ret[$source[0]] = $columns;
                }
            }
        }

        return $ret;
    }

    /**
     * TODO
     */
    function columns2String($hash)
    {
        foreach ($hash as $source => $columns) {
            if (isset($string)) {
                $string .= "\n";
            } else {
                $string = '';
            }
            $line = '';
            foreach ($columns as $column) {
                if (!empty($line)) {
                    $line .= "\t";
                }
                $line .= $column;
            }
        }

        return $string;
    }

    /**
     * Checks if a user has the specified permissions on the current backend.
     *
     * @param string $filter                What are we checking for.
     * @param optional integer $permission  What permission to check for.
     *
     * @return boolean  Returns true if the user has permission, false if
     *                  they do not.
     */
    function checkPermissions($filter, $permission = PERMS_READ)
    {
        $userID = Auth::getAuth();

        switch ($filter) {
        case 'backend':
            $backendTag = 'gollem:backends:' . $_SESSION['gollem']['backend_key'];
            if (!$GLOBALS['perms']->exists($backendTag) ||
                $GLOBALS['perms']->hasPermission($backendTag, $userID, $permission)) {
                return true;
            }
            break;
        }

        return false;
    }

    /**
     * TODO
     */
    function permissionsFilter($backends, $filter, $permission = PERMS_READ)
    {
        $out = array();
        $userID = Auth::getAuth();

        switch ($filter) {
        case 'backend':
            foreach ($backends as $backendID => $name) {
                $backendTag = 'gollem:backends:' . $backendID;
                if (!$GLOBALS['perms']->exists($backendTag) ||
                    $GLOBALS['perms']->hasPermission($backendTag, $userID, $permission)) {
                    $out[$backendID] = $name;
                }
            }
            break;

        default:
            $out = $backends;
        }

        return $out;
    }

    /**
     * Produces a directory link used for navigation.
     *
     * @access public
     *
     * @param string $currdir  The current directory string.
     * @param string $url      The URL to link to.
     *
     * @return string  The directory navigation string.
     */
    function directoryNavLink($currdir, $url)
    {
        $label = array();
        $root_dir = Gollem::getRoot();
        if ($currdir == $root_dir) {
            $label[] = '[' . _("Root") . ']';
        } else {
            $parts = explode('/', $currdir);
            $parts_count = count($parts);

            $label[] = Horde::link(Util::addParameter($url, 'dir', $root_dir), sprintf(_("Up to %s"), _("Root")), 'header') . '[' . _("Root") . ']</a>';

            for ($i = 1; $i <= $parts_count; $i++) {
                $part = array_slice($parts, 0, $i);
                $dir = implode('/', $part);
                if ((empty($root_dir) || strstr($dir, $root_dir) !== false) &&
                    ($root_dir != $dir)) {
                    if ($i == $parts_count) {
                        $label[] = $parts[($i - 1)];
                    } else {
                        $label[] = Horde::link(Util::addParameter($url, 'dir', $dir), sprintf(_("Up to %s"), $dir), 'header') . $parts[($i - 1)] . '</a>';
                    }
                }
            }
        }

        return implode('/', $label);
    }

    /**
     * Build Gollem's list of menu items.
     *
     * @access public
     */
    function getMenu($returnType = 'object')
    {
        require_once 'Horde/Menu.php';

        $menu = &new Menu(HORDE_MENU_MASK_ALL & ~HORDE_MENU_MASK_LOGIN);
        $menu->add(Util::addParameter(Horde::applicationUrl('manager.php'), 'dir', Gollem::getHome()), _("Home"), 'home.png');

        if (strpos($_SERVER['PHP_SELF'], 'manager.php') !== false) {
            if (Gollem::checkPermissions('backend', PERMS_EDIT)) {
                $menu->add('', _("Create Folder"), 'mkdir.png', null, '', 'createFolder(); return false;');
            }
            if (Gollem::checkPermissions('backend', PERMS_READ)) {
                $menu->add('', _("Change Folder"), 'cd.png', null, '', 'changeDirectory(); return false;');
            }
        }

        /* Logout. */
        $can_autologin = Gollem::canAutoLogin();
        if (!$can_autologin || !$GLOBALS['prefs']->getValue('show_sidebar')) {
            if (!$GLOBALS['prefs']->getValue('show_sidebar') ||
                (Auth::getProvider() == 'gollem')) {
                $logout_url = Horde::getServiceLink('logout', 'horde');
                $menu->add($logout_url, _("_Log out"), 'logout.png', $GLOBALS['registry']->getImageDir('horde'), $GLOBALS['conf']['menu']['always'] ? 'horde_main' : '_parent');
            } else {
                $logout_url = Horde::getServiceLink('logout', 'gollem');
                $menu->add($logout_url, _("_Log out"), 'logout.png', $GLOBALS['registry']->getImageDir('horde'));
            }

        }
        if ($GLOBALS['conf']['backend']['backend_list'] == 'shown') {
            $logout_url = Auth::addLogoutParameters(Horde::applicationUrl('login.php'), AUTH_REASON_LOGOUT);
            $menu->add($logout_url, _("_Change Server"), 'logout.png', $GLOBALS['registry']->getImageDir('horde'));
        }

        if ($returnType == 'object') {
            return $menu;
        } else {
            return $menu->render();
        }
    }

}
